local L = AceLibrary("AceLocale-2.2"):new("Talented")

function Talented:UpdateCurrentTemplate()
	
	local template = self.current or {}
	template.name = "Current"
	local total = UnitCharacterPoints("player") 
	for tab, talents in ipairs(self.talents) do
		template[tab] = { talents = {} }
		for index, info in ipairs(talents.infos) do
			local _, _, _, _, rank = GetTalentInfo(tab, index)
			template[tab][index] = rank
			total = total + rank
		end
	end
	self.maxpoints = total
	self.current = template
end

function Talented:UpdateView()
	if not self.base then return end
	local template
	if self.mode == "apply" then
		template = self.current
	else
		template = self.template
	end
	local total = 0
		
	for tab, talents in ipairs(self.talents) do
		local count = 0
		for index, info in ipairs(talents.infos) do
			local rank = (template[tab] and template[tab][index]) or 0
			count = count + rank
			local button = info.button
			local color = GRAY_FONT_COLOR
			local state = self:GetTalentState(template, tab, index)
			if state == "empty" and self.mode == "view" then
				state = "unavailable"
			end
			if state == "unavailable" then
				button.texture:SetDesaturated(1)
				button.slot:SetVertexColor(0.65, 0.65, 0.65)
				button.rank:Hide()
				button.rank.texture:Hide()
			else
				button.rank:Show()
				button.rank.texture:Show()
				button.rank:SetText(rank)
				button.texture:SetDesaturated(0)
				if state == "full" then
					color = NORMAL_FONT_COLOR
				else
					color = GREEN_FONT_COLOR
				end
				button.slot:SetVertexColor(color.r, color.g, color.b)
				button.rank:SetVertexColor(color.r, color.g, color.b)
			end
			if info.reqs then
				for _, req in ipairs(info.reqs) do
					local ecolor = color
					if ecolor == GREEN_FONT_COLOR then
						if self.mode == "edit" then
							local s = self:GetTalentState(template, tab, req.source.index)
							if s ~= "full" then
								ecolor = RED_FONT_COLOR
							end
						else
							ecolor = NORMAL_FONT_COLOR
						end
					end
					for _, element in ipairs(req.elements) do
						element:SetVertexColor(ecolor.r, ecolor.g, ecolor.b)
					end
				end
			end
		end
		talents.frame.name:SetText(string.format(L["%s (%d)"], talents.name, count))
		total = total + count
		local clear = talents.frame.clear
		if self.mode ~= "edit" or count <= 0 or self.template == self.current then
			clear:Hide()
		else
			clear:Show()
		end
	end
	local points = self.base.points
	local maxpoints = self.maxpoints
	points:SetText(string.format(L["%d/%d"], total, maxpoints))
	local color
	if total < maxpoints then
		color = GREEN_FONT_COLOR
	elseif total > maxpoints then
		color = RED_FONT_COLOR
	else
		color = NORMAL_FONT_COLOR
	end
	points:SetTextColor(color.r, color.g, color.b)
	local edit = self.base.editname
	edit:SetText(template.name)
	if template == self.current then
		color = NORMAL_FONT_COLOR
	else
		color = GREEN_FONT_COLOR
	end
	edit:SetTextColor(color.r, color.g, color.b)
end
